@@ -56,7 +56,7 @@ import java.util.concurrent.atomic.AtomicInteger;
  * track of deltas during a request.
  *
  * @author Filip Hanik
- * @version $Revision: 896383 $ $Date: 2010-01-06 11:51:53 +0100 (Mi, 06. Jan 2010) $
+ * @version $Id: DeltaSession.java 1153318 2011-08-03 01:34:02Z kfujino $
  */
 
 public class DeltaSession extends StandardSession implements Externalizable,ClusterSession,ReplicatedMapEntry {
@@ -98,10 +98,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
     // ----------------------------------------------------------- Constructors
 
     /**
-     * Construct a new Session associated with the specified Manager.
-     *
-     * @param manager
-     *            The manager with which this Session is associated
+     * Construct a new Session.
      */
     public DeltaSession() {
         this(null);
@@ -237,17 +234,17 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         this.isPrimarySession = primarySession;
     }
 
+
     /**
-     * Set the session identifier for this session without notify listeners.
-     *
-     * @param id
-     *            The new session identifier
+     * {@inheritDoc}
      */
-    public void setIdInternal(String id) {
-        this.id = id;
+    @Override
+    public void setId(String id, boolean notify) {
+        super.setId(id, notify);
         resetDeltaRequest();
     }
 
+
     /**
      * Set the session identifier for this session.
      *
@@ -337,6 +334,28 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         }
     }
 
+    /**
+     * Set the authentication type used to authenticate our cached
+     * Principal, if any.
+     *
+     * @param authType The new cached authentication type
+     */
+    @Override
+    public void setAuthType(String authType) {
+        setAuthType(authType, true);
+    }
+
+    public void setAuthType(String authType, boolean addDeltaRequest) {
+        try { 
+            lock();
+            super.setAuthType(authType);
+            if (addDeltaRequest && (deltaRequest != null))
+                deltaRequest.setAuthType(authType);
+        } finally {
+            unlock();
+        }
+    }
+
     /**
      * Return the <code>isValid</code> flag for this session.
      */
@@ -399,7 +418,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
             return;
         String expiredId = getIdInternal();
 
-        if(expiredId != null && manager != null &&
+        if(notifyCluster && expiredId != null && manager != null &&
            manager instanceof DeltaManager) {
             DeltaManager dmanager = (DeltaManager)manager;
             CatalinaCluster cluster = dmanager.getCluster();
@@ -595,7 +614,9 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         try {
             lock();
             super.setAttribute(name,value, notify);
-            if (addDeltaRequest && (deltaRequest != null)) deltaRequest.setAttribute(name, value);
+            if (addDeltaRequest && deltaRequest != null && !exclude(name)) {
+                deltaRequest.setAttribute(name, value);
+            }
         } finally {
             unlock();
         }
@@ -771,7 +792,9 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
             if (value == null) return;
 
             super.removeAttributeInternal(name,notify);
-            if (addDeltaRequest && (deltaRequest != null)) deltaRequest.removeAttribute(name);
+            if (addDeltaRequest && deltaRequest != null && !exclude(name)) {
+                deltaRequest.removeAttribute(name);
+            }
 
         }finally {
             unlock();
