@@ -42,7 +42,7 @@ import org.apache.catalina.util.RequestUtil;
  * use XSLT, that is unnecessarily complex.
  *
  * @author Peter Lin
- * @version $Id: StatusTransformer.java 939518 2010-04-30 00:08:58Z kkolinko $
+ * @version $Id: StatusTransformer.java 1057270 2011-01-10 16:43:40Z jfclere $
  */
 
 public class StatusTransformer {
@@ -575,7 +575,7 @@ public class StatusTransformer {
                 }
 
                 writer.print("<a href=\"#" + (count++) + ".0\">");
-                writer.print(webModuleName);
+                writer.print(filter(webModuleName));
                 writer.print("</a>");
                 if (iterator.hasNext()) {
                     writer.print("<br>");
@@ -650,7 +650,7 @@ public class StatusTransformer {
             }
 
             writer.print("<h1>");
-            writer.print(name);
+            writer.print(filter(name));
             writer.print("</h1>");
             writer.print("</a>");
 
@@ -778,11 +778,11 @@ public class StatusTransformer {
                 mBeanServer.invoke(objectName, "findMappings", null, null);
             
             writer.print("<h2>");
-            writer.print(servletName);
+            writer.print(filter(servletName));
             if ((mappings != null) && (mappings.length > 0)) {
                 writer.print(" [ ");
                 for (int i = 0; i < mappings.length; i++) {
-                    writer.print(mappings[i]);
+                    writer.print(filter(mappings[i]));
                     if (i < mappings.length - 1) {
                         writer.print(" , ");
                     }
@@ -871,10 +871,20 @@ public class StatusTransformer {
         }
 
         if (mb) {
+            StringBuilder buff = new StringBuilder();
+            if (bytes < 0) {
+                buff.append('-');
+                bytes = -bytes;
+            }
             long mbytes = bytes / (1024 * 1024);
             long rest = 
                 ((bytes - (mbytes * (1024 * 1024))) * 100) / (1024 * 1024);
-            return (mbytes + "." + ((rest < 10) ? "0" : "") + rest + " MB");
+            buff.append(mbytes).append('.');
+            if (rest < 10) {
+                buff.append('0');
+            }
+            buff.append(rest).append(" MB");
+            return buff.toString();
         } else {
             return ((bytes / 1024) + " KB");
         }
