@@ -64,7 +64,7 @@ import org.xml.sax.SAXParseException;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Id: ContextConfig.java 939353 2010-04-29 15:50:43Z kkolinko $
+ * @version $Id: ContextConfig.java 964731 2010-07-16 10:12:01Z markt $
  */
 
 public class ContextConfig
@@ -717,7 +717,7 @@ public class ContextConfig
      */
     protected void contextConfig() {
         
-        // Open the default web.xml file, if it exists
+        // Open the default context.xml file, if it exists
         if( defaultContextXml==null && context instanceof StandardContext ) {
             defaultContextXml = ((StandardContext)context).getDefaultContextXml();
         }
@@ -725,7 +725,11 @@ public class ContextConfig
         if( defaultContextXml==null ) getDefaultContextXml();
 
         if (!context.getOverride()) {
-            processContextConfig(new File(getBaseDir()), defaultContextXml);
+            File defaultContextFile = new File(defaultContextXml);
+            if (!defaultContextFile.isAbsolute()) {
+                defaultContextFile =new File(getBaseDir(), defaultContextXml);
+            }
+            processContextConfig(defaultContextFile.getParentFile(), defaultContextFile.getName());
             processContextConfig(getConfigBase(), getHostConfigPath(Constants.HostContextXml));
         }
         if (context.getConfigFile() != null)
