@@ -21,13 +21,17 @@ package org.apache.catalina.valves;
 
 import java.io.BufferedWriter;
 import java.io.File;
-import java.io.FileWriter;
+import java.io.FileOutputStream;
 import java.io.IOException;
+import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
+import java.io.UnsupportedEncodingException;
 import java.net.InetAddress;
+import java.nio.charset.Charset;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Date;
+import java.util.Enumeration;
 import java.util.List;
 import java.util.TimeZone;
 
@@ -35,6 +39,7 @@ import javax.servlet.ServletException;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpSession;
 
+import org.apache.catalina.AccessLog;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleListener;
@@ -45,6 +50,7 @@ import org.apache.catalina.util.StringManager;
 import org.apache.coyote.RequestInfo;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.buf.B2CConverter;
 
 
 /**
@@ -115,12 +121,10 @@ import org.apache.juli.logging.LogFactory;
  * @author Takayuki Kaneko
  * @author Peter Rossbach
  * 
- * @version $Id: AccessLogValve.java 939353 2010-04-29 15:50:43Z kkolinko $
+ * @version $Id: AccessLogValve.java 1156552 2011-08-11 09:17:29Z markt $
  */
 
-public class AccessLogValve
-    extends ValveBase
-    implements Lifecycle {
+public class AccessLogValve extends ValveBase implements AccessLog, Lifecycle {
 
     private static Log log = LogFactory.getLog(AccessLogValve.class);
 
@@ -301,7 +305,14 @@ public class AccessLogValve
      * Date format to place in log file name. Use at your own risk!
      */
     protected String fileDateFormat = null;
-    
+
+    /**
+     * Character set used by the log file. If it is <code>null</code>, the
+     * system default character set will be used. An empty string will be
+     * treated as <code>null</code> when this property is assigned.
+     */
+    protected String encoding = null;
+
     /**
      * Array of AccessLogElement, they will be used to make log message.
      */
@@ -522,6 +533,29 @@ public class AccessLogValve
         this.fileDateFormat =  fileDateFormat;
     }
 
+    /**
+     * Return the character set name that is used to write the log file.
+     *
+     * @return Character set name, or <code>null</code> if the system default
+     *  character set is used.
+     */
+    public String getEncoding() {
+        return encoding;
+    }
+
+    /**
+     * Set the character set that is used to write the log file.
+     * 
+     * @param encoding The name of the character set.
+     */
+    public void setEncoding(String encoding) {
+        if (encoding != null && encoding.length() > 0) {
+            this.encoding = encoding;
+        } else {
+            this.encoding = null;
+        }
+    }
+
     // --------------------------------------------------------- Public Methods
 
     /**
@@ -556,25 +590,30 @@ public class AccessLogValve
     
             long t2 = System.currentTimeMillis();
             long time = t2 - t1;
-    
-            if (logElements == null || condition != null
-                    && null != request.getRequest().getAttribute(condition)) {
-                return;
-            }
-    
-            Date date = getDate();
-            StringBuffer result = new StringBuffer(128);
-    
-            for (int i = 0; i < logElements.length; i++) {
-                logElements[i].addElement(result, date, request, response, time);
-            }
-    
-            log(result.toString());
+            
+            log(request, response, time);
         } else
             getNext().invoke(request, response);       
     }
 
     
+    public void log(Request request, Response response, long time) {
+        if (logElements == null || condition != null
+                && null != request.getRequest().getAttribute(condition)) {
+            return;
+        }
+
+        Date date = getDate();
+        StringBuffer result = new StringBuffer(128);
+
+        for (int i = 0; i < logElements.length; i++) {
+            logElements[i].addElement(result, date, request, response, time);
+        }
+
+        log(result.toString());
+    }
+
+
     /**
      * Rename the existing log file to something else. Then open the
      * old log file name up once again. Intended to be called by a JMX
@@ -592,7 +631,7 @@ public class AccessLogValve
             try {
                 holder.renameTo(new File(newFileName));
             } catch (Throwable e) {
-                log.error("rotate failed", e);
+                log.error(sm.getString("accessLogValve.rotateFail"), e);
             }
 
             /* Make sure date is correct */
@@ -662,7 +701,7 @@ public class AccessLogValve
                     try {
                         close();
                     } catch (Throwable e) {
-                        log.info("at least this wasn't swallowed", e);
+                        log.info(sm.getString("accessLogValve.closeFail"), e);
                     }
 
                     /* Make sure date is correct */
@@ -712,26 +751,51 @@ public class AccessLogValve
         File dir = new File(directory);
         if (!dir.isAbsolute())
             dir = new File(System.getProperty("catalina.base"), directory);
-        dir.mkdirs();
+        if (!dir.exists()) {
+            if (!dir.mkdirs()) {
+                log.error(sm.getString("accessLogValve.openDirFail", dir));
+            }
+        }
 
         // Open the current log file
-        try {
-            String pathname;
-            // If no rotate - no need for dateStamp in fileName
-            if (rotatable) {
-                pathname = dir.getAbsolutePath() + File.separator + prefix
-                        + dateStamp + suffix;
-            } else {
-                pathname = dir.getAbsolutePath() + File.separator + prefix
-                        + suffix;
+        File pathname;
+        // If no rotate - no need for dateStamp in fileName
+        if (rotatable) {
+            pathname = new File(dir.getAbsoluteFile(), prefix + dateStamp
+                    + suffix);
+        } else {
+            pathname = new File(dir.getAbsoluteFile(), prefix + suffix);
+        }
+        File parent = pathname.getParentFile();
+        if (!parent.exists()) {
+            if (!parent.mkdirs()) {
+                log.error(sm.getString("accessLogValve.openDirFail", parent));
             }
-            writer = new PrintWriter(new BufferedWriter(new FileWriter(
-                    pathname, true), 128000), false);
-            
-            currentLogFile = new File(pathname);
+        }
+
+        Charset charset = null;
+        if (encoding != null) {
+            try {
+                charset = B2CConverter.getCharset(encoding);
+            } catch (UnsupportedEncodingException ex) {
+                log.error(sm.getString(
+                        "accessLogValve.unsupportedEncoding", encoding), ex);
+            }
+        }
+        if (charset == null) {
+            charset = Charset.defaultCharset();
+        }
+
+        try {
+            writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(
+                    new FileOutputStream(pathname, true), charset), 128000),
+                    false);
+
+            currentLogFile = pathname;
         } catch (IOException e) {
             writer = null;
             currentLogFile = null;
+            log.error(sm.getString("accessLogValve.openFail", pathname), e);
         }
     }
  
@@ -1209,12 +1273,15 @@ public class AccessLogValve
 
         public void addElement(StringBuffer buf, Date date, Request request,
                 Response response, long time) {
-            String value = request.getHeader(header);
-            if (value == null) {
-                buf.append('-');
-            } else {
-                buf.append(value);
+            Enumeration<String> iter = request.getHeaders(header);
+            if (iter.hasMoreElements()) {
+                buf.append(iter.nextElement());
+                while (iter.hasMoreElements()) {
+                    buf.append(',').append(iter.nextElement());
+                }
+                return;
             }
+            buf.append('-');
         }
     }
 
@@ -1389,7 +1456,7 @@ public class AccessLogValve
     /**
      * create an AccessLogElement implementation which needs header string
      */
-    private AccessLogElement createAccessLogElement(String header, char pattern) {
+    protected AccessLogElement createAccessLogElement(String header, char pattern) {
         switch (pattern) {
         case 'i':
             return new HeaderElement(header);
@@ -1409,7 +1476,7 @@ public class AccessLogValve
     /**
      * create an AccessLogElement implementation
      */
-    private AccessLogElement createAccessLogElement(char pattern) {
+    protected AccessLogElement createAccessLogElement(char pattern) {
         switch (pattern) {
         case 'a':
             return new RemoteAddrElement();
