@@ -22,6 +22,7 @@ import java.io.IOException;
 import java.io.InterruptedIOException;
 import java.net.InetAddress;
 import java.nio.ByteBuffer;
+import java.security.NoSuchProviderException;
 import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
 
@@ -334,6 +335,20 @@ public class AjpAprProcessor implements ActionHook {
     public void setRequiredSecret(String requiredSecret) { this.requiredSecret = requiredSecret; }
 
 
+    /**
+     * When client certificate information is presented in a form other than
+     * instances of {@link java.security.cert.X509Certificate} it needs to be
+     * converted before it can be used and this property controls which JSSE
+     * provider is used to perform the conversion. For example it is used with
+     * the AJP connectors, the HTTP APR connector and with the
+     * {@link org.apache.catalina.valves.SSLValve}. If not specified, the
+     * default provider will be used. 
+     */
+    protected String clientCertProvider = null;
+    public String getClientCertProvider() { return clientCertProvider; }
+    public void setClientCertProvider(String s) { this.clientCertProvider = s; }
+
+    
     // --------------------------------------------------------- Public Methods
 
 
@@ -555,8 +570,13 @@ public class AjpAprProcessor implements ActionHook {
                             certData.getLength());
                 // Fill the  elements.
                 try {
-                    CertificateFactory cf =
-                        CertificateFactory.getInstance("X.509");
+                    CertificateFactory cf;
+                    if (clientCertProvider == null) {
+                        cf = CertificateFactory.getInstance("X.509");
+                    } else {
+                        cf = CertificateFactory.getInstance("X.509",
+                                clientCertProvider);
+                    }
                     while(bais.available() > 0) {
                         X509Certificate cert = (X509Certificate)
                             cf.generateCertificate(bais);
@@ -573,6 +593,9 @@ public class AjpAprProcessor implements ActionHook {
                 } catch (java.security.cert.CertificateException e) {
                     log.error(sm.getString("ajpprocessor.certs.fail"), e);
                     return;
+                } catch (NoSuchProviderException e) {
+                    log.error(sm.getString("ajpprocessor.certs.fail"), e);
+                    return;
                 }
                 request.setAttribute(AprEndpoint.CERTIFICATE_KEY, jsseCerts);
             }
@@ -1226,7 +1249,7 @@ public class AjpAprProcessor implements ActionHook {
     protected void flush()
         throws IOException {
         if (outputBuffer.position() > 0) {
-            if (Socket.sendbb(socket, 0, outputBuffer.position()) < 0) {
+            if ((socket != 0) && Socket.sendbb(socket, 0, outputBuffer.position()) < 0) {
                 throw new IOException(sm.getString("ajpprocessor.failedsend"));
             }
             outputBuffer.clear();
