@@ -31,6 +31,7 @@ import org.apache.tomcat.util.res.StringManager;
 import org.apache.coyote.ActionCode;
 import org.apache.coyote.OutputBuffer;
 import org.apache.coyote.Response;
+import org.apache.coyote.http11.filters.GzipOutputFilter;
 
 /**
  * Output buffer.
@@ -94,6 +95,12 @@ public class InternalAprOutputBuffer
     protected static StringManager sm =
         StringManager.getManager(Constants.Package);
 
+    /**
+     * Logger.
+     */
+    private static final org.apache.juli.logging.Log log
+        = org.apache.juli.logging.LogFactory.getLog(
+                InternalAprOutputBuffer.class);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -280,6 +287,19 @@ public class InternalAprOutputBuffer
 
         }
 
+        // go through the filters and if there is gzip filter
+        // invoke it to flush
+        for (int i = 0; i <= lastActiveFilter; i++) {
+            if (activeFilters[i] instanceof GzipOutputFilter) {
+                if (log.isDebugEnabled()) {
+                    log.debug("Flushing the gzip filter at position " + i +
+                            " of the filter chain...");
+                }
+                ((GzipOutputFilter) activeFilters[i]).flush();
+                break;
+            }
+        }
+        
         // Flush the current buffer
         flushBuffer();
 
